@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     LUdet.cpp
- Revision: $Id: LUdet.cpp 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: LUdet.cpp 171 2010-10-04 13:57:19Z kulshres $
  Contents: example for
              * Computation of the determinant of a matrix
                by LU-decomposition of the system matrix without pivoting 
@@ -22,7 +22,7 @@
 
 /****************************************************************************/
 /*                                                                 INCLUDES */
-#include <../examples/additional_examples/lufact/LU.h>
+#include "LU.h"
 
 
 /****************************************************************************/
