@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     sparse_jacobian.cpp
- Revision: $Id: sparse_jacobian.cpp 157 2010-09-02 12:14:24Z kulshres $
+ Revision: $Id: sparse_jacobian.cpp 203 2011-02-23 11:33:40Z kulshres $
  Contents: example for computation of sparse jacobians
 
  Copyright (c) Andrea Walther, Andreas Griewank, Andreas Kowarz, 
@@ -17,8 +17,8 @@
 #include <cstdlib>
 #include <cstdio>
 
-#include <adolc.h>
-#include <adolc_sparse.h>
+#include <adolc/adolc.h>
+#include <adolc/adolc_sparse.h>
 
 #define tag 1
 
@@ -93,6 +93,9 @@ int main() {
     for (i=0;i<nnz;i++)
         printf("%2d %2d %10.6f\n\n",rind[i],cind[i],values[i]);
 
+    free(rind); rind=NULL;
+    free(cind); cind=NULL;
+    free(values); values=NULL;
 /*--------------------------------------------------------------------------*/
 /*  same approach but using row compression                                 */
 /*--------------------------------------------------------------------------*/
@@ -107,6 +110,9 @@ int main() {
     for (i=0;i<nnz;i++)
         printf("%2d %2d %10.6f\n\n",rind[i],cind[i],values[i]);
 
+    free(rind); rind=NULL;
+    free(cind); cind=NULL;
+    free(values); values=NULL;
 /*--------------------------------------------------------------------------*/
 /*  change value of x, but not the sparsity pattern                         */
 /*--------------------------------------------------------------------------*/
@@ -129,7 +135,9 @@ int main() {
     for (i=0;i<nnz;i++)
         printf("%2d %2d %10.6f\n\n",rind[i],cind[i],values[i]);
 
-
+    free(rind); rind=NULL;
+    free(cind); cind=NULL;
+    free(values); values=NULL;
 /*--------------------------------------------------------------------------*/
 /*  same approach but using row compression                                 */
 /*--------------------------------------------------------------------------*/
@@ -144,6 +152,9 @@ int main() {
     for (i=0;i<nnz;i++)
         printf("%2d %2d %10.6f\n\n",rind[i],cind[i],values[i]);
 
+    free(rind); rind=NULL;
+    free(cind); cind=NULL;
+    free(values); values=NULL;
 /****************************************************************************/
 /*******       sparse Jacobians, separate drivers             ***************/
 /****************************************************************************/
