@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     adouble.cpp
- Revision: $Id: adouble.cpp 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: adouble.cpp 198 2011-02-09 16:27:25Z kulshres $
  Contents: adouble.C contains that definitions of procedures used to 
            define various badouble, adub, and adouble operations. 
            These operations actually have two purposes.
@@ -20,9 +20,9 @@
    
 ----------------------------------------------------------------------------*/
 
-#include <adouble.h>
-#include <oplate.h>
-#include <taping_p.h>
+#include <adolc/adouble.h>
+#include <adolc/oplate.h>
+#include "taping_p.h"
 
 using namespace std;
 
@@ -32,15 +32,16 @@ using namespace std;
 /*--------------------------------------------------------------------------*/
 void condassign( double &res, const double &cond,
                  const double &arg1, const double &arg2 ) {
-    res = cond ? arg1 : arg2;
+    res = cond > 0 ? arg1 : arg2;
 }
 
 /*--------------------------------------------------------------------------*/
 void condassign( double &res, const double &cond,
                  const double &arg) {
-    res = cond ? arg : res;
+    res = cond > 0 ? arg : res;
 }
 
+#if !defined(_ISOC99_SOURCE) && !defined(__USE_ISOC99)
 /*--------------------------------------------------------------------------*/
 double fmax( const double &x, const double &y ) {
     if (y > x)
@@ -56,6 +57,7 @@ double fmin( const double &x, const double &y ) {
     else
         return x;
 }
+#endif
 
 /*--------------------------------------------------------------------------*/
 /* The remaining routines define the badouble, adub and adouble routines.   */
@@ -153,26 +155,17 @@ adouble::adouble( const adub& a ) {
 /****************************************************************************/
 /*                                                              DESTRUCTORS */
 
-#define ADOLC_FREE_LOC(X) \
-    {\
-        ADOLC_OPENMP_THREAD_NUMBER;\
-        ADOLC_OPENMP_GET_THREAD_NUMBER;\
-        ++ADOLC_GLOBAL_TAPE_VARS.numToFree;\
-        if (X < ADOLC_GLOBAL_TAPE_VARS.minLocToFree)\
-            ADOLC_GLOBAL_TAPE_VARS.minLocToFree = X;\
-    }
-
 /*--------------------------------------------------------------------------*/
 adouble::~adouble() {
 #ifdef overwrite
-    ADOLC_FREE_LOC(location);
+    free_loc(location);
 #endif
 }
 
 /*--------------------------------------------------------------------------*/
 adub::~adub() {
 #ifdef overwrite
-    ADOLC_FREE_LOC(location);
+    free_loc(location);
 #endif
 }
 
@@ -338,10 +331,6 @@ badouble& badouble::operator = ( const adub& a ) {
         revreal tempVal = ADOLC_GLOBAL_TAPE_VARS.store[a_loc];
         if (ADOLC_CURRENT_TAPE_INFOS.keepTaylors)
             ADOLC_OVERWRITE_SCAYLOR(ADOLC_GLOBAL_TAPE_VARS.store[location],&ADOLC_GLOBAL_TAPE_VARS.store[a_loc]);
-        if (a_loc == ADOLC_GLOBAL_TAPE_VARS.locMinUnused-1) {
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused--;     // The temporary will die in a minute and
-            ADOLC_GLOBAL_TAPE_VARS.numToFree--;         // by reducing dealloc and current_top
-        }                    // we neutralize that effect
         ADOLC_GLOBAL_TAPE_VARS.store[location] = tempVal;
     } else {
         if (ADOLC_CURRENT_TAPE_INFOS.traceFlag) { // old: write_assign_a(location,a_loc);
@@ -586,10 +575,6 @@ badouble& badouble::operator += ( const adub& a ) {
         ADOLC_GLOBAL_TAPE_VARS.store[location] += ADOLC_GLOBAL_TAPE_VARS.store[a_loc];
         if (ADOLC_CURRENT_TAPE_INFOS.keepTaylors)
             ADOLC_DELETE_SCAYLOR(&ADOLC_GLOBAL_TAPE_VARS.store[a_loc]);
-        if (a_loc == ADOLC_GLOBAL_TAPE_VARS.locMinUnused-1) {
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused--;     // The temporary will die in a minute and
-            ADOLC_GLOBAL_TAPE_VARS.numToFree--;         // by reducing dealloc and current_top
-        }                    // we neutralize that effect
         --ADOLC_CURRENT_TAPE_INFOS.numTays_Tape;
     } else {
         if (ADOLC_CURRENT_TAPE_INFOS.traceFlag) { // old: write_assign_a(location,a_loc);
@@ -640,10 +625,6 @@ badouble& badouble::operator -= ( const adub& a ) {
         ADOLC_GLOBAL_TAPE_VARS.store[location] -= ADOLC_GLOBAL_TAPE_VARS.store[a_loc];
         if (ADOLC_CURRENT_TAPE_INFOS.keepTaylors)
             ADOLC_DELETE_SCAYLOR(&ADOLC_GLOBAL_TAPE_VARS.store[a_loc]);
-        if (a_loc == ADOLC_GLOBAL_TAPE_VARS.locMinUnused-1) {
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused--;     // The temporary will die in a minute and
-            ADOLC_GLOBAL_TAPE_VARS.numToFree--;         // by reducing dealloc and current_top
-        }                    // we neutralize that effect
         --ADOLC_CURRENT_TAPE_INFOS.numTays_Tape;
     } else {
         if (ADOLC_CURRENT_TAPE_INFOS.traceFlag) { // old: write_assign_a(location,a_loc);
