@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     sparse/sparsedrivers.cpp
- Revision: $Id: sparsedrivers.cpp 157 2010-09-02 12:14:24Z kulshres $
+ Revision: $Id: sparsedrivers.cpp 204 2011-03-22 14:11:28Z awalther $
  Contents: "Easy To Use" C++ interfaces of SPARSE package
  
  Copyright (c) Andrea Walther
@@ -143,9 +143,9 @@ void generate_seed_jac
 /*                                                                           */
 
 int hess_pat(
-    short        tag,        /* tape identification                        */
-    int          indep,      /* number of independent variables            */
-    double       *basepoint, /* independant variable values                */
+    short          tag,        /* tape identification                        */
+    int            indep,      /* number of independent variables            */
+    const double  *basepoint,  /* independant variable values                */
     unsigned int **crs,
     /* returned compressed row block-index storage                         */
     int          option
@@ -209,6 +209,18 @@ void generate_seed_hess
 }
 #endif
 
+static void deepcopy_HP(unsigned int ***HPnew, unsigned int **HP, int indep)
+{
+    int i,j,s;
+    *HPnew = (unsigned int **)malloc(indep*sizeof(unsigned int *));
+    for (i=0; i<indep; i++) {
+       s=HP[i][0];
+       (*HPnew)[i] = (unsigned int *)malloc((s+1)*(sizeof(unsigned int)));
+       for (j=0; j<=s; j++)
+           (*HPnew)[i][j] = HP[i][j];
+    }
+}
+
 /****************************************************************************/
 /*******       sparse Jacobians, complete driver              ***************/
 /****************************************************************************/
@@ -365,10 +377,27 @@ int sparse_jac(
 
     /* recover compressed Jacobian => ColPack library */
  
+    if (*values != NULL && *rind != NULL && *cind != NULL) {
+      // everything is preallocated, we assume correctly
+      // call usermem versions
       if (options[3] == 1)
-       jr1d->RecoverD2Row_CoordinateFormat(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+       jr1d->RecoverD2Row_CoordinateFormat_usermem(g, sJinfos.B, sJinfos.JP, rind, cind, values);
      else
-       jr1d->RecoverD2Cln_CoordinateFormat(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+       jr1d->RecoverD2Cln_CoordinateFormat_usermem(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+    } else {
+      // at least one of rind cind values is not allocated, deallocate others
+      // and call unmanaged versions
+      if (*values != NULL)
+	  free(*values);
+      if (*rind != NULL)
+	  free(*rind);
+      if (*cind != NULL)
+	  free(*cind);
+      if (options[3] == 1)
+       jr1d->RecoverD2Row_CoordinateFormat_unmanaged(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+     else
+       jr1d->RecoverD2Cln_CoordinateFormat_unmanaged(g, sJinfos.B, sJinfos.JP, rind, cind, values);
+    }
 
     return ret_val;
 
@@ -388,7 +417,7 @@ int sparse_hess(
     short          tag,        /* tape identification                     */
     int            indep,      /* number of independent variables         */
     int            repeat,     /* indicated repeated call with same seed  */
-    double        *basepoint,  /* independant variable values             */
+    const double  *basepoint,  /* independant variable values             */
     int           *nnz,        /* number of nonzeros                      */
     unsigned int **rind,       /* row index                               */
     unsigned int **cind,       /* column index                            */
@@ -442,7 +471,11 @@ int sparse_hess(
 	  {
 	    tapeInfos=getTapeInfos(tag);
 	    memcpy(&ADOLC_CURRENT_TAPE_INFOS, tapeInfos, sizeof(TapeInfos));
-	    sHinfos.HP     = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.HP;
+            if (indep != ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.indep) {
+                fprintf(DIAG_OUT,"ADOL-C Error: wrong number of independents stored in hessian pattern.\n");
+                exit(-1);
+            }
+	    deepcopy_HP(&sHinfos.HP,ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.HP,indep);	    
 	  }
 
 	sHinfos.indep = indep;
@@ -470,8 +503,7 @@ int sparse_hess(
 	  g->GenerateSeedHessian(&Seed, &dummy, &sHinfos.p, 
 		  	         "SMALLEST_LAST","STAR"); 
 
-	
-	sHinfos.Hcomp = myalloc2(indep,sHinfos.p);
+       	sHinfos.Hcomp = myalloc2(indep,sHinfos.p);
         sHinfos.Xppp = myalloc3(indep,sHinfos.p,1);
 
 	for (i=0; i<indep; i++)
@@ -491,9 +523,12 @@ int sparse_hess(
 
 	sHinfos.g = (void *) g;
 	sHinfos.hr = (void *) hr;
+
 	setTapeInfoHessSparse(tag, sHinfos);
+
 	tapeInfos=getTapeInfos(tag);
 	memcpy(&ADOLC_CURRENT_TAPE_INFOS, tapeInfos, sizeof(TapeInfos));
+
     }
     else
       {
@@ -511,7 +546,6 @@ int sparse_hess(
 	hr = (HessianRecovery *)ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.hr;
       }
 
-
     if (sHinfos.Upp == NULL) {
         printf(" ADOL-C error in sparse_hess():"
                " First call with repeat = 0 \n");
@@ -570,11 +604,27 @@ int sparse_hess(
 //      else
 //        HessianRecovery::DirectRecover_CoordinateFormat(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
  
+    if (*values != NULL && *rind != NULL && *cind != NULL) {
+     // everything is preallocated, we assume correctly
+     // call usermem versions
      if (options[1] == 0)
-       hr->IndirectRecover_CoordinateFormat(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
+       hr->IndirectRecover_CoordinateFormat_usermem(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
      else
-       hr->DirectRecover_CoordinateFormat(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
- 
+       hr->DirectRecover_CoordinateFormat_usermem(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
+    } else {
+      // at least one of rind cind values is not allocated, deallocate others
+      // and call unmanaged versions
+      if (*values != NULL)
+	  free(*values);
+      if (*rind != NULL)
+	  free(*rind);
+      if (*cind != NULL)
+	  free(*cind);
+     if (options[1] == 0)
+       hr->IndirectRecover_CoordinateFormat_unmanaged(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
+     else
+       hr->DirectRecover_CoordinateFormat_unmanaged(g, sHinfos.Hcomp, sHinfos.HP, rind, cind, values);
+    }
     return ret_val;
 
 }
@@ -587,29 +637,59 @@ int sparse_hess(
 
 
 /****************************************************************************/
-/*******        sparse Hessians, complete driver              ***************/
+/*******      sparse Hessians, set and get sparsity pattern   ***************/
 /****************************************************************************/
 
 void set_HP(
     short          tag,        /* tape identification                     */
     int            indep,      /* number of independent variables         */
     unsigned int ** HP)
+#ifdef SPARSE
 {
     SparseHessInfos sHinfos;
     TapeInfos *tapeInfos;
 
     tapeInfos=getTapeInfos(tag);
     memcpy(&ADOLC_CURRENT_TAPE_INFOS, tapeInfos, sizeof(TapeInfos));
-    sHinfos.nnz_in = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.nnz_in;
-    sHinfos.HP     = HP;
-    sHinfos.Hcomp  = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.Hcomp;
-    sHinfos.Xppp   = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.Xppp;
-    sHinfos.Yppp   = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.Yppp;
-    sHinfos.Zppp   = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.Zppp;
-    sHinfos.Upp    = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.Upp;
-    sHinfos.p      = ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.p;
+    sHinfos.nnz_in = 0;
+    deepcopy_HP(&sHinfos.HP,HP,indep);
+    sHinfos.Hcomp  = NULL;
+    sHinfos.Xppp   = NULL;
+    sHinfos.Yppp   = NULL;
+    sHinfos.Zppp   = NULL;
+    sHinfos.Upp    = NULL;
+    sHinfos.p      = 0;
+    sHinfos.g      = NULL;
+    sHinfos.hr     = NULL;
+    sHinfos.indep  = indep;
     setTapeInfoHessSparse(tag, sHinfos);
 }
+#else
+{
+    fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if sparse configuration option was used\n", __FUNCTION__);
+    exit(-1);
+}
+#endif
+
+void get_HP(
+    short          tag,        /* tape identification                     */
+    int            indep,      /* number of independent variables         */
+    unsigned int *** HP)
+#ifdef SPARSE
+{
+    SparseHessInfos sHinfos;
+    TapeInfos *tapeInfos;
+
+    tapeInfos=getTapeInfos(tag);
+    memcpy(&ADOLC_CURRENT_TAPE_INFOS, tapeInfos, sizeof(TapeInfos));
+    deepcopy_HP(HP,ADOLC_CURRENT_TAPE_INFOS.pTapeInfos.sHinfos.HP,indep);
+}
+#else
+{
+    fprintf(DIAG_OUT, "ADOL-C error: function %s can only be used if sparse configuration option was used\n", __FUNCTION__);
+    exit(-1);
+}
+#endif
 
 /*****************************************************************************/
 /*                                                    JACOBIAN BLOCK PATTERN */
@@ -1000,18 +1080,20 @@ void freeSparseHessInfos(double **Hcomp, double ***Xppp, double ***Yppp, double
 
    if(Zppp)
       myfree3(Zppp);
+
    if(Upp)
       myfree2(Upp);
-    for (int i=0;i<indep;i++) {
-      free(HP[i]);
-    }
 
-    free(HP);
+   if(HP)
+     {
+       for (int i=0;i<indep;i++) 
+   	 free(HP[i]);
+       free(HP);
+     }
 
 #ifdef HAVE_LIBCOLPACK
      if (g) 
        delete (GraphColoringInterface *) g;
-
     if (hr)
 	delete (HessianRecovery*) hr;
 #endif
