@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     taping.c
- Revision: $Id: taping.c 116 2010-07-20 12:29:32Z awalther $
+ Revision: $Id: taping.c 230 2011-06-08 12:12:19Z kulshres $
  Contents: all C functions directly accessing at least one of the four tapes
            (operations, locations, constants, value stack)
 
@@ -20,6 +20,19 @@
 #include <adolc/oplate.h>
 #include "taping_p.h"
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#if defined(_WINDOWS) && !__STDC__
+#define stat _stat
+#define S_IFDIR _S_IFDIR
+#define S_IFMT _S_IFMT
+#endif
+
+#ifndef S_ISDIR
+#define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
+#endif
+
 /*--------------------------------------------------------------------------*/
 /* Tape identification (ADOLC & version check) */
 ADOLC_ID adolc_id;
@@ -292,19 +305,20 @@ void printError() {
             break;
         default:
             fprintf(DIAG_OUT, ">>> ");
-            fprintf(DIAG_OUT, strerror(errno));
+            fprintf(DIAG_OUT, "%s", strerror(errno));
             fprintf(DIAG_OUT, " <<<\n");
             break;
     }
 }
 
 /* the base names of every tape type */
-const char *tapeBaseNames[] = {
-            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_LOCATIONS_NAME,
-            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_VALUES_NAME,
-            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_OPERATIONS_NAME,
-            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_TAYLORS_NAME
-        };
+char *tapeBaseNames[4];
+
+void clearTapeBaseNames() {
+    int i;
+    for(i=0;i<4;i++)
+	free(tapeBaseNames[i]);
+}
 
 /****************************************************************************/
 /* The subroutine get_fstr appends to the tape base name of type tapeType   */
@@ -391,8 +405,20 @@ char *createFileName(short tapeID, int tapeType) {
 void readConfigFile() {
     FILE *configFile = NULL;
     char inputLine[ADOLC_LINE_LENGTH + 1];
-    char *pos1 = NULL, *pos2 = NULL, *pos3 = NULL, *pos4 = NULL, *end = NULL;
+    char *pos1 = NULL, *pos2 = NULL, *pos3 = NULL, *pos4 = NULL, *start = NULL, *end = NULL;
+    int base;
     long int number = 0;
+    char *path = NULL;
+    int defdirsize = strlen(TAPE_DIR PATHSEPARATOR);
+    tapeBaseNames[0] = strdup(
+	TAPE_DIR PATHSEPARATOR ADOLC_LOCATIONS_NAME);
+    tapeBaseNames[1] = strdup(
+	TAPE_DIR PATHSEPARATOR ADOLC_VALUES_NAME);
+    tapeBaseNames[2] = strdup(
+	TAPE_DIR PATHSEPARATOR ADOLC_OPERATIONS_NAME);
+    tapeBaseNames[3] = strdup(
+	TAPE_DIR PATHSEPARATOR ADOLC_TAYLORS_NAME);
+
     ADOLC_OPENMP_THREAD_NUMBER;
     ADOLC_OPENMP_GET_THREAD_NUMBER;
 
@@ -428,10 +454,70 @@ void readConfigFile() {
                     fprintf(DIAG_OUT, "ADOL-C warning: Malformed input line "
                             "in .adolcrc ignored!\n");
             } else {
-                number = strtol(pos3 + 1, &end, 10);
-                if (end == (pos3 + 1)) {
-                    fprintf(DIAG_OUT, "ADOL-C warning: Unable to parse number in "
-                            ".adolcrc!\n");
+		if (*(pos3 + 1) == '0' && (*(pos3 + 2) == 'x' || *(pos3 + 2) == 'X')) {
+		    start = pos3 + 3;
+		    base = 16;
+		} else if (*(pos3 + 1) == '0') {
+		    start = pos3 + 2;
+		    base = 8;
+		} else {
+		    start = pos3 + 1;
+		    base = 10;
+		}
+		number = strtol(start, &end, base);
+                if (end == start) {
+		    *pos2 = 0;
+		    *pos4 = 0;
+		    if (strcmp(pos1 + 1, "TAPE_DIR") == 0) {
+			struct stat st;
+			int err;
+			path = pos3 + 1;
+			err = stat(path,&st);
+			if (err == 0 && S_ISDIR(st.st_mode)) {
+			    int pathlen, pathseplen, namelen[4];
+			    int i;
+			    pathlen=strlen(path);
+			    pathseplen=strlen(PATHSEPARATOR);
+			    for(i = 0; i < 4; i++)
+				namelen[i] = strlen(tapeBaseNames[i]);
+			    clearTapeBaseNames();
+			    for(i = 0; i < 4; i++) {
+				char *currpos;
+				int fnamelen;
+				tapeBaseNames[i] = (char*)calloc(namelen[i] - defdirsize + pathlen + pathseplen + 1, sizeof(char));
+				currpos = tapeBaseNames[i];
+				strncpy(currpos,path,pathlen);
+				currpos += pathlen;
+				strncpy(currpos,PATHSEPARATOR,pathseplen);
+				currpos += pathseplen;
+				switch (i) {
+				case 0:
+				    fnamelen = strlen(ADOLC_LOCATIONS_NAME);
+				    strncpy(currpos,ADOLC_LOCATIONS_NAME,fnamelen);
+				    break;
+				case 1:
+				    fnamelen = strlen(ADOLC_VALUES_NAME);
+				    strncpy(currpos,ADOLC_VALUES_NAME,fnamelen);
+				    break;
+				case 2:
+				    fnamelen = strlen(ADOLC_OPERATIONS_NAME);
+				    strncpy(currpos,ADOLC_OPERATIONS_NAME,fnamelen);
+				    break;
+				case 3:
+				    fnamelen = strlen(ADOLC_TAYLORS_NAME);
+				    strncpy(currpos,ADOLC_TAYLORS_NAME,fnamelen);
+				    break;
+				}
+				currpos += fnamelen;
+				*currpos = '\0';
+			    }
+			    fprintf(DIAG_OUT, "ADOL-C info: using TAPE_DIR %s for all disk bound tapes\n",path);
+			} else
+			    fprintf(DIAG_OUT, "ADOL-C warning: TAPE_DIR %s in .adolcrc is not an existing directory,\n will continue using %s for writing tapes\n", path, TAPE_DIR);
+		    }
+		    else 
+			fprintf(DIAG_OUT, "ADOL-C warning: Unable to parse number in "
+				".adolcrc!\n");
                 } else {
                     *pos2 = 0;
                     *pos4 = 0;
@@ -479,15 +565,18 @@ static unsigned int numTBuffersInUse = 0;
 /* record all existing adoubles on the tape
  * - intended to be used in start_trace only */
 void take_stock() {
-    locint space_left, vals_left, loc = 0;
+    locint space_left, loc = 0;
     double *vals;
+    size_t vals_left;
     ADOLC_OPENMP_THREAD_NUMBER;
 
     ADOLC_OPENMP_GET_THREAD_NUMBER;
     space_left  = get_val_space(); /* remaining space in const. tape buffer */
-    vals_left = ADOLC_GLOBAL_TAPE_VARS.locMinUnused;
+    vals_left = ADOLC_GLOBAL_TAPE_VARS.storeSize;
     vals      = ADOLC_GLOBAL_TAPE_VARS.store;
-
+    
+    /* if we have adoubles in use */
+    if (ADOLC_GLOBAL_TAPE_VARS.numLives > 0) {
     /* fill the current values (real) tape buffer and write it to disk
      * - do this as long as buffer can be fully filled */
     while (space_left < vals_left) {
@@ -508,6 +597,7 @@ void take_stock() {
         ADOLC_PUT_LOCINT(loc);
         put_vals_notWriteBlock(vals, vals_left);
     }
+    }
     ADOLC_CURRENT_TAPE_INFOS.traceFlag = 1;
 }
 
@@ -520,124 +610,28 @@ locint keep_stock() {
     ADOLC_OPENMP_THREAD_NUMBER;
     ADOLC_OPENMP_GET_THREAD_NUMBER;
     /* if we have adoubles in use */
-    if (ADOLC_GLOBAL_TAPE_VARS.numMaxAlive > 0) {
-        locint loc2 = ADOLC_GLOBAL_TAPE_VARS.numMaxAlive - 1;
+    if (ADOLC_GLOBAL_TAPE_VARS.numLives > 0) {
+        locint loc2 = ADOLC_GLOBAL_TAPE_VARS.storeSize - 1;
 
         /* special signal -> all alive adoubles recorded on the end of the
          * value stack -> special handling at the beginning of reverse */
         put_op(death_not);
-        ADOLC_PUT_LOCINT(0);    /* lowest loc */
+        ADOLC_PUT_LOCINT(1);    /* lowest loc */
         ADOLC_PUT_LOCINT(loc2); /* highest loc */
 
-        ADOLC_CURRENT_TAPE_INFOS.numTays_Tape += ADOLC_GLOBAL_TAPE_VARS.numMaxAlive;
+        ADOLC_CURRENT_TAPE_INFOS.numTays_Tape += ADOLC_GLOBAL_TAPE_VARS.storeSize - 1;
         /* now really do it if keepTaylors ist set */
         if (ADOLC_CURRENT_TAPE_INFOS.keepTaylors) {
             do {
                 ADOLC_WRITE_SCAYLOR(ADOLC_GLOBAL_TAPE_VARS.store[loc2]);
-            } while (loc2-- > 0);
+            } while (--loc2 > 0);
         }
     }
     ADOLC_CURRENT_TAPE_INFOS.traceFlag = 0;
-    return ADOLC_GLOBAL_TAPE_VARS.numMaxAlive;
+    return ADOLC_GLOBAL_TAPE_VARS.storeSize;
 }
 
 
-void updateLocs() {
-    ADOLC_OPENMP_THREAD_NUMBER;
-    ADOLC_OPENMP_GET_THREAD_NUMBER;
-
-    /* deallocate dead adoubles if they form a contiguous tail */
-    #ifdef overwrite
-    if (ADOLC_GLOBAL_TAPE_VARS.numToFree &&
-            ADOLC_GLOBAL_TAPE_VARS.minLocToFree + ADOLC_GLOBAL_TAPE_VARS.numToFree ==
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused) {
-        ADOLC_GLOBAL_TAPE_VARS.locMinUnused = ADOLC_GLOBAL_TAPE_VARS.minLocToFree ;
-        ADOLC_GLOBAL_TAPE_VARS.numToFree = 0;
-        ADOLC_GLOBAL_TAPE_VARS.minLocToFree = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-    }
-    #endif
-}
-
-/****************************************************************************/
-/* Returns the next free location in "adouble" memory.                      */
-/****************************************************************************/
-locint next_loc() {
-    locint newStoreSize;
-    ADOLC_OPENMP_THREAD_NUMBER;
-    ADOLC_OPENMP_GET_THREAD_NUMBER;
-
-    updateLocs();
-    if (ADOLC_GLOBAL_TAPE_VARS.locMinUnused == ADOLC_GLOBAL_TAPE_VARS.numMaxAlive)
-        ++ADOLC_GLOBAL_TAPE_VARS.numMaxAlive;
-    if (ADOLC_GLOBAL_TAPE_VARS.numMaxAlive > ADOLC_GLOBAL_TAPE_VARS.storeSize) {
-        /* try to double (plus 2) the available space */
-        ++ADOLC_GLOBAL_TAPE_VARS.storeSize;
-        if (ADOLC_GLOBAL_TAPE_VARS.storeSize == ADOLC_GLOBAL_TAPE_VARS.maxLoc) {
-            failAdditionalInfo1 = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-            fail(ADOLC_TAPING_TO_MANY_LOCINTS);
-        }
-        failAdditionalInfo3 = ADOLC_GLOBAL_TAPE_VARS.storeSize;
-        newStoreSize = ADOLC_GLOBAL_TAPE_VARS.storeSize * 2;
-        ADOLC_GLOBAL_TAPE_VARS.storeSize = newStoreSize;
-        failAdditionalInfo4 = newStoreSize;
-        failAdditionalInfo5 = ADOLC_GLOBAL_TAPE_VARS.store;
-        if (ADOLC_GLOBAL_TAPE_VARS.store == NULL) {
-            ADOLC_GLOBAL_TAPE_VARS.store =
-                (double *)malloc(ADOLC_GLOBAL_TAPE_VARS.storeSize * sizeof(double));
-            ADOLC_GLOBAL_TAPE_VARS.minLocToFree = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-        } else
-            ADOLC_GLOBAL_TAPE_VARS.store =
-                (double *)realloc((char *)ADOLC_GLOBAL_TAPE_VARS.store,
-                        ADOLC_GLOBAL_TAPE_VARS.storeSize * sizeof(double));
-        if (ADOLC_GLOBAL_TAPE_VARS.store == NULL)
-            fail(ADOLC_TAPING_STORE_REALLOC_FAILED);
-    }
-    return ADOLC_GLOBAL_TAPE_VARS.locMinUnused++;
-}
-
-/****************************************************************************/
-/* Returns the next #size free locations in "adouble" memory.               */
-/****************************************************************************/
-locint next_loc_v(int size) {
-    locint newStoreSize, retVal;
-    ADOLC_OPENMP_THREAD_NUMBER;
-    ADOLC_OPENMP_GET_THREAD_NUMBER;
-
-    updateLocs();
-    if (ADOLC_GLOBAL_TAPE_VARS.locMinUnused + size > ADOLC_GLOBAL_TAPE_VARS.numMaxAlive)
-        ADOLC_GLOBAL_TAPE_VARS.numMaxAlive = ADOLC_GLOBAL_TAPE_VARS.locMinUnused + size;
-    while (ADOLC_GLOBAL_TAPE_VARS.numMaxAlive > ADOLC_GLOBAL_TAPE_VARS.storeSize) {
-        /* try to double (plus 2) the available space */
-        ++ADOLC_GLOBAL_TAPE_VARS.storeSize;
-        if (ADOLC_GLOBAL_TAPE_VARS.storeSize*2 > ADOLC_GLOBAL_TAPE_VARS.maxLoc)
-            newStoreSize = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-        else newStoreSize = ADOLC_GLOBAL_TAPE_VARS.storeSize * 2;
-        if (newStoreSize == ADOLC_GLOBAL_TAPE_VARS.maxLoc) {
-            failAdditionalInfo3 = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-            fail(ADOLC_TAPING_TO_MANY_LOCINTS);
-        } else {
-            ADOLC_GLOBAL_TAPE_VARS.storeSize = newStoreSize;
-            if (ADOLC_GLOBAL_TAPE_VARS.storeSize == 2) {
-                ADOLC_GLOBAL_TAPE_VARS.store =
-                    (double *)malloc(ADOLC_GLOBAL_TAPE_VARS.storeSize * sizeof(double));
-                ADOLC_GLOBAL_TAPE_VARS.minLocToFree = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-            } else
-                ADOLC_GLOBAL_TAPE_VARS.store =
-                    (double *)realloc((char *)ADOLC_GLOBAL_TAPE_VARS.store,
-                            ADOLC_GLOBAL_TAPE_VARS.storeSize * sizeof(double));
-            if (ADOLC_GLOBAL_TAPE_VARS.store == NULL)
-                fail(ADOLC_TAPING_STORE_REALLOC_FAILED);
-        }
-    }
-    #if defined(ADOLC_DEBUG)
-    fprintf (DIAG_OUT, "ADOL-C debug: Top is: %d !\n",
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused + size);
-    #endif
-    retVal = ADOLC_GLOBAL_TAPE_VARS.locMinUnused;
-    ADOLC_GLOBAL_TAPE_VARS.locMinUnused += size;
-    return retVal;
-}
-
 /****************************************************************************/
 /* Set up statics for writing taylor data                                   */
 /****************************************************************************/
@@ -1058,16 +1052,6 @@ void start_trace() {
         fail(ADOLC_MORE_STAT_SPACE_REQUIRED);
     for (i = 0; i < statSpace; ++i) ADOLC_PUT_LOCINT(0);
 
-    /* free locations if possible => take_stock may benefit */
-    if (ADOLC_GLOBAL_TAPE_VARS.minLocToFree +
-            ADOLC_GLOBAL_TAPE_VARS.numToFree ==
-            ADOLC_GLOBAL_TAPE_VARS.locMinUnused) {
-        ADOLC_GLOBAL_TAPE_VARS.locMinUnused =
-            ADOLC_GLOBAL_TAPE_VARS.minLocToFree ;
-        ADOLC_GLOBAL_TAPE_VARS.numToFree = 0;
-        ADOLC_GLOBAL_TAPE_VARS.minLocToFree = ADOLC_GLOBAL_TAPE_VARS.maxLoc;
-    }
-
     /* initialize value stack if necessary */
     if (ADOLC_CURRENT_TAPE_INFOS.keepTaylors)
         taylor_begin(ADOLC_CURRENT_TAPE_INFOS.stats[TAY_BUFFER_SIZE], NULL, 0);
@@ -1089,7 +1073,7 @@ void stop_trace(int flag) {
     ADOLC_CURRENT_TAPE_INFOS.stats[NUM_DEPENDENTS] =
         ADOLC_CURRENT_TAPE_INFOS.numDeps;
     ADOLC_CURRENT_TAPE_INFOS.stats[NUM_MAX_LIVES] =
-        ADOLC_GLOBAL_TAPE_VARS.numMaxAlive;
+        ADOLC_GLOBAL_TAPE_VARS.storeSize;
 
     taylor_close(ADOLC_CURRENT_TAPE_INFOS.stats[TAY_BUFFER_SIZE]);
 
