@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     taping_p.h
- Revision: $Id: taping_p.h 157 2010-09-02 12:14:24Z kulshres $
+ Revision: $Id: taping_p.h 199 2011-02-22 13:25:56Z kulshres $
  Contents: declarations for used by taping routines
  
  Copyright (c) Andreas Kowarz
@@ -17,6 +17,9 @@
 #include <adolc/common.h>
 #include <adolc/taping.h>
 #include <errno.h>
+#ifdef __cplusplus
+#include "storemanager.h"
+#endif
 
 BEGIN_C_DECLS
 
@@ -264,14 +267,11 @@ typedef struct TapeInfos {
 }
 TapeInfos;
 
-typedef struct {
+typedef struct GlobalTapeVarsCL {
     double* store;              /* double store for calc. while taping */
+    size_t storeSize;
+    size_t numLives;
     locint maxLoc;
-    locint locMinUnused;        /* largest live location + 1 */
-    locint numMaxAlive;         /* maximal # of lives so far */
-    locint storeSize;           /* current size of store */
-    locint numToFree;           /* # of locations to be freed */
-    locint minLocToFree;        /* lowest loc to be freed */
 
     locint operationBufferSize; /* Defaults to the value specified in */
     locint locationBufferSize;  /* usrparms.h. May be overwritten by values */
@@ -283,11 +283,18 @@ typedef struct {
     char newTape;               /* signals: at least one tape created (0/1) */
     char branchSwitchWarning;
     TapeInfos *currentTapeInfosPtr;
+#ifdef __cplusplus
+    StoreManager *storeManagerPtr;
+    GlobalTapeVarsCL();
+    ~GlobalTapeVarsCL();
+#else
+    void *storeManagerPtr;
+#endif
 }
 GlobalTapeVars;
 
 #if defined(_OPENMP)
-
+#error nicht hier
 extern int isParallel();
 
 #define ADOLC_TAPE_INFOS_BUFFER_DECL *tapeInfosBuffer
@@ -388,8 +395,8 @@ void updateLocs();
 locint next_loc();
 /* returns the next free location in "adouble" memory */
 
-locint next_loc_v(int size);
-/* returns the next #size free locations in "adouble" memory */
+void free_loc(locint loc);
+/* frees the specified location in "adouble" memory */
 
 void taylor_begin(uint bufferSize, double **Tg, int degreeSave);
 /* set up statics for writing taylor data */
@@ -612,6 +619,9 @@ double get_val_r();
 /* tries to read a local config file containing, e.g., buffer sizes */
 void readConfigFile();
 
+/* clear the tapeBaseNames that were alocated above in readConfigFile() */
+void clearTapeBaseNames();
+
 /****************************************************************************/
 /* This function sets the flag "newTape" if either a taylor buffer has been */
 /* created or a taping process has been performed. Calling the function is  */
